from qgis.core import QgsRectangle, QgsVectorLayer
from qgis.gui import QgsMapTool


class MouseClick(QgsMapTool):

    def __init__(self, canvas, callback):
        super().__init__(canvas)
        self.canvas = canvas
        self.callback = callback

    def canvasReleaseEvent(self, event):
        layers = self.canvas.layers()
        p = self.toMapCoordinates(event.pos())
        w = self.canvas.mapUnitsPerPixel() * 10
        try:
            rect = QgsRectangle(p.x() - w, p.y() - w, p.x() + w, p.y() + w)
        except:
            return

        for layer in layers:
            if isinstance(layer, QgsVectorLayer) and layer.isSpatial():
                fields = [field.name().upper() for field in layer.fields()]
                if 'IMAGEPATH' in fields:
                    lRect = self.canvas.mapSettings().mapToLayerCoordinates(layer, rect)
                    layer.selectByRect(lRect)
                    selected_features = layer.selectedFeatures()
                    if selected_features != []:
                        feature = selected_features[0]
                        self.callback(layer, feature)
