from qgis.core import (
    QgsVectorLayer, QgsFeature, QgsField,
    QgsGeometry, QgsPointXY, QgsExpressionContextUtils
)
from qgis.PyQt.QtCore import QVariant

from ..core.error import Error


class Creator():

    main_group_name = "Scolopax"

    def __init__(self, qgis_project, scolopax_project):
        self.qgis_project = qgis_project
        self.scolopax_project = scolopax_project

        self.main_layer_group = None
        self.project_layer_group = None
        self.create_scolopax_group()
        self.create_project_group()

    def create_scolopax_group(self):
        root = self.qgis_project.layerTreeRoot()
        self.main_layer_group = root.findGroup(self.main_group_name)
        if not self.main_layer_group:
            self.main_layer_group = root.addGroup(self.main_group_name)

    def create_project_group(self):
        if self.main_layer_group.findGroup(self.scolopax_project.name):
            raise Error(f"Группа слоёв проекта \"{self.scolopax_project.name}\" уже существует.")

        self.project_layer_group = self.main_layer_group.addGroup(self.scolopax_project.name)

    def create_drive_layer(self, drive):
        layer = QgsVectorLayer("Point?crs=EPSG:4326", drive.name, 'memory')
        provider = layer.dataProvider()
        provider.addAttributes([
            QgsField("Index", QVariant.Int),
            QgsField("Lat", QVariant.Double),
            QgsField("Lng", QVariant.Double),
            QgsField("Time", QVariant.String),
            QgsField("Dist", QVariant.Double),
            QgsField("RDist", QVariant.Double),
            QgsField("RSDist", QVariant.Double),
            QgsField("Address", QVariant.String),
            QgsField("Image", QVariant.String),
            QgsField("ImagePath", QVariant.String)
        ])
        layer.updateFields()

        QgsExpressionContextUtils.setLayerVariable(layer, 'camera_yaw', drive.camera_yaw)
        QgsExpressionContextUtils.setLayerVariable(layer, 'camera_pitch', drive.camera_pitch)
        QgsExpressionContextUtils.setLayerVariable(layer, 'camera_roll', drive.camera_roll)

        for point in drive.data['stitchedData']:
            feature = QgsFeature()
            feature.setGeometry(QgsGeometry.fromPointXY(QgsPointXY(point['lng'], point['lat'])))

            image = point.get('image', "")
            image_path = ""
            if image:
                image_path = str(drive.path / "images" / point['image'])

            address = None
            if 'linearAddress' in point:
                km = point['linearAddress']['kilometer']
                local_distance = point['linearAddress']['localDistance']
                address = f"{km}+{round(local_distance):03}"

            feature.setAttributes([
                point['index'],
                point['lat'],
                point['lng'],
                point.get('time'),
                point.get("distance"),
                point.get("roadDistance",),
                point.get("roadSectionDistance"),
                address,
                image,
                image_path
            ])
            provider.addFeature(feature)

        layer.updateExtents()
        self.qgis_project.addMapLayer(layer, False)
        self.project_layer_group.addLayer(layer)
