from pathlib import Path
import json


class ScolopaxProject():

    def __init__(self, project_file_path):
        self.project_file_path = Path(project_file_path)

        self.id = None
        self.name = None
        self.drives = None

        self.load()

    def load(self):
        with open(self.project_file_path, encoding="utf-8") as f:
            data = json.load(f)
            self.id = data['id']
            self.name = data['name']
            self.drives = [{
                'id': d['id'],
                'name': d['name'],
                'camera_yaw': d.get('cameraYaw', 0),
                'camera_pitch': d.get('cameraPitch', 0),
                'camera_roll': d.get('cameraRoll', 0)
            } for d in data['drives']]

    @property
    def project_dir_path(self):
        return self.project_file_path.parent

    def get_drive_path(self, drive_id):
        return self.project_dir_path / drive_id

