import React from 'react';
import moment from 'moment';


class Info extends React.Component {

  state = {};

  componentDidMount = () => {
    this.interval = setInterval(this.fetchQgisData, 100);
  }

  componentWillUnmount = () => {
    this.interval = null;
  }

  fetchQgisData = () => {
    const data = {
      driveName: window.qgisApi.getDriveName(),
      index: window.qgisApi.getIndex(),
      imagePath: window.qgisApi.getImagePath(),
      lat: window.qgisApi.getLat(),
      lng: window.qgisApi.getLng(),
      time: window.qgisApi.getTime(),
      address: window.qgisApi.getAddress(),
      distance: window.qgisApi.getDistance(),
      roadDistance: window.qgisApi.getRoadDistance(),
      roadSectionDistance: window.qgisApi.getRoadSectionDistance(),
      windowSize: window.qgisApi.getWindowSize()
    };

    this.setState({ ...data });
  }

  getDriveName = () => {
    return this.state.driveName;
  }

  getIndex = () => {
    return this.state.index;
  }

  getLat = () => {
    return Number(this.state.lat).toFixed(6);
  }

  getLng = () => {
    return Number(this.state.lng).toFixed(6);
  }

  getTime = () => {
    const time = moment(this.state.time, "YYYY-MM-DDTHH:mm:ssZ");
    return time.format("DD-MM-YYYY HH:mm:ss");
  }

  getAddress = () => {
    return this.state.address;
  }

  getRoadDistance = () => {
    return Number(this.state.roadDistance).toFixed(0);
  }

  getRoadSectionDistance = () => {
    return Number(this.state.roadSectionDistance).toFixed(0);
  }

  getDistance = () => {
    return Number(this.state.distance).toFixed(0);
  }

  render() {
    return (
      <div
        style={{ fontSize: (this.state.windowSize == "S" ? 13 : 17) + "px" }}
      >
        <div className="row">
          <div className="col-10">
            <b>{this.getDriveName()}</b>
          </div>
          <div className="col-2">
            # {this.getIndex()}
          </div>
        </div>

        <div className="row">
          <div className="col-4">
            {this.state.windowSize != "S" ? "Лин. адрес: " : ""}{this.getAddress()}
          </div>
          <div className="col-4">
            {this.state.windowSize != "S" ? "Коорд.: " : ""}{this.getLat()};{this.getLng()}
          </div>
          <div className="col-4">
            {this.state.windowSize != "S" ? "Время: ": ""}{this.getTime()}
          </div>
        </div>

        <div className="row">
          <div className="col-4">
            Дорога: {this.getRoadDistance()} м
          </div>
          <div className="col-4">
            Участок: {this.getRoadSectionDistance()} м
          </div>
          <div className="col-4">
            Трек: {this.getDistance()} м
          </div>
        </div>
      </div>
    );
  }
}

export default Info;
