# -*- coding: utf-8 -*-
"""
/***************************************************************************
 Scolopax
                                 A QGIS plugin
 View Scolopax projects in QGIS
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2022-01-20
        git sha              : $Format:%H$
        copyright            : (C) 2022 by Цириус
        email                : office@cirius.su
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
import os.path
import time

from qgis.PyQt.QtCore import QSettings, QTranslator, QCoreApplication
from qgis.PyQt.QtGui import QIcon
from qgis.PyQt.QtWidgets import QAction
from qgis.core import Qgis, QgsProject

from .resources import *
from .ui.scolopax_import_dialog import ScolopaxImportDialog
from .ui.scolopax_webview import ScolopaxWebView
from .imports.importer import Importer
from .core.error import Error
from .core.mouse_click import MouseClick


class Scolopax:
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface
        self.canvas = self.iface.mapCanvas()
        self.project = QgsProject.instance()
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'Scolopax_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)
            QCoreApplication.installTranslator(self.translator)

        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&Scolopax')

        # Check if plugin was started the first time in current QGIS session
        # Must be set in initGui() to survive plugin reloads
        self.first_start = None

        self.webview_window_size = "M"
        self.webview_position = None

    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('Scolopax', message)

    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None
    ):
        """Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        """

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            # Adds plugin icon to Plugins toolbar
            self.iface.addToolBarIcon(action)

        if add_to_menu:
            self.iface.addPluginToMenu(
                self.menu,
                action)

        self.actions.append(action)

        return action

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        icon_path = ':/plugins/scolopax/icons/icon.png'
        self.add_action(
            icon_path,
            text=self.tr(u'Scolopax'),
            callback=self.run,
            parent=self.iface.mainWindow())

        icon_path = ':/plugins/scolopax/icons/panorama.png'
        self.add_action(
            icon_path,
            text="Панорама 360",
            callback=self.set_mouse_click_map_tool,
            parent=self.iface.mainWindow())

        # will be set False in run()
        self.first_start = True

    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for action in self.actions:
            self.iface.removePluginMenu(
                self.tr(u'&Scolopax'),
                action)
            self.iface.removeToolBarIcon(action)

    def show_webview(self, layer, feature):
        self.webview = ScolopaxWebView(
            self,
            layer,
            feature
        )
        self.webview.show()

    def exec(self):
        time_start = time.time()

        scolopax_project_file_path = self.get_scolopax_project_file_path()
        importer = Importer(self.project, scolopax_project_file_path)
        importer.run()

        time_exec = round(time.time() - time_start)

        self.push_ok_message(time_exec)

    def push_ok_message(self, time_exec):
        self.iface.messageBar().pushMessage(
            "Импорт проекта Scolopax выполнен",
            f"время: {time_exec}с",
            level=Qgis.Success,
            duration=0
        )

    def push_error_message(self, message):
        self.iface.messageBar().pushMessage(
            "Ошибка",
            message,
            level=Qgis.Warning,
            duration=0
        )

    def get_scolopax_project_file_path(self):
        return self.dlg.projectFilePath.filePath()

    def set_mouse_click_map_tool(self):
        tool = MouseClick(self.canvas, callback=self.panorama_point_selected)
        self.canvas.setMapTool(tool)

    def panorama_point_selected(self, layer, feature):
        self.iface.setActiveLayer(layer)
        self.show_webview(layer, feature)

    def set_webview_window_size(self, window_size):
        if window_size not in ("S", "M", "L"):
            raise Exception(f"Не удалось установить размер окна \"{window_size}\".")

        self.webview_window_size = window_size

    def run(self):
        if self.first_start:
            self.first_start = False
            self.dlg = ScolopaxImportDialog()

        self.dlg.show()

        result = self.dlg.exec_()
        if result:
            try:
                self.exec()
            except Error as e:
                self.push_error_message(str(e))
